use log::{info, trace};

pub mod fetch;
pub mod identifier;
pub mod installer;

pub struct Package {
    pub identifier: identifier::PackageIdentifier,
    pub pkgfile: pkgfile::PKGFile,
    is_installed: bool,
    is_indexed: bool,
}

impl Package {
    /// Create a new package from a package identifier and a package file.
    pub fn new(identifier: identifier::PackageIdentifier, pkgfile: pkgfile::PKGFile) -> Package {
        Package {
            identifier,
            pkgfile,
            is_installed: false,
            is_indexed: false,
        }
    }

    /// Fetch a package from a package identifier.
    pub fn fetch(
        package_identifier: identifier::PackageIdentifier,
    ) -> Result<Package, fetch::FetchError> {
        match &package_identifier {
            identifier::PackageIdentifier::Path(path) => {
                trace!("fetching package from path: {}", path);
                let pkgfile = fetch::fetch_by_path(path).unwrap();
                Ok(Package::new(package_identifier, pkgfile))
            }
            identifier::PackageIdentifier::URI(url) => {
                trace!("fetching package from uri: {}", url);
                let pkgfile = fetch::fetch_by_uri(url).unwrap();
                Ok(Package::new(package_identifier, pkgfile))
            }
            identifier::PackageIdentifier::PackageLocator(locator) => {
                trace!("fetching package from locator: {}", locator);
                let pkgfile = fetch::fetch_by_package_locator(locator.clone()).unwrap();
                Ok(Package::new(package_identifier, pkgfile))
            }
        }
    }

    /// Get the package manifest.
    pub fn manifest(&self) -> manifest::Manifest {
        manifest::Manifest::try_from(self.pkgfile.manifest.clone()).unwrap()
    }

    /// Install the package.
    pub fn install(&mut self, build: bool) -> Result<(), installer::errors::InstallError> {
        let manifest = self.manifest();
        let mut installer = installer::PackageInstaller::new(
            manifest.clone(),
            self.pkgfile.clone(),
            if build {
                info!("building package: {}...", if self.manifest().package.name.is_empty() {
                    self.identifier.to_string()
                } else {
                    self.manifest().package.name
                });
                installer::InstallType::Build
            } else {
                info!("using binary install for package: {}...", if self.manifest().package.name.is_empty() {
                    self.identifier.to_string()
                } else {
                    self.manifest().package.name
                });
                installer::InstallType::Bin
            },
        );
        match installer.install() {
            Ok(_) => {
                self.is_installed = true;
                Ok(())
            }
            Err(e) => Err(e),
        }
    }

    /// Check if the package is installed.
    pub fn is_installed(&self) -> bool {
        self.is_installed
    }

    /// Check if the package is indexed.
    pub fn is_indexed(&self) -> bool {
        self.is_indexed
    }

    /// Remove the package from the system.
    pub fn remove(&mut self) -> Result<(), ()> {
        unimplemented!();
    }
}
